library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

--------------------------------------------------------------------------------
-- Take the time difference output by OnePPSCounter.Vhd, extract the sign and
-- magnitude information for display, and throw out the trailing zeros to have
-- the magnitude in units of clock cycles.
--
-- Outputs both a signed difference in two's complement (MSB is sign) and the
-- magnitude and sign information (for the Digital I/O display).
--
-- Ilya Belopolski
-- January 2012
--------------------------------------------------------------------------------

entity CalculateLittleDiff is port
   (
     CLK            : in    std_logic;
     COUNT          : in    std_logic_vector(31 downto 0);
     DIFF           : out   std_logic_vector(31 downto 0);
     SIGNED_DIFF    : out   std_logic_vector(31 downto 0);
     SIGN           : out   std_logic
   );
end CalculateLittleDiff;


architecture ark of CalculateLittleDiff is

     signal diffdiff     : signed(31 downto 0);
     signal off          : signed(31 downto 0);
     signal offdiff      : std_logic_vector(31 downto 0);
     signal signdiff     : std_logic_vector(31 downto 0);

begin

     diffdiff <= signed(COUNT) when rising_edge(CLK);

     signdiff <= std_logic_vector(diffdiff);
     SIGNED_DIFF <= signdiff(5 downto 0) & signdiff(31 downto 6);

     findDiff : process (CLK) is
     begin
          if rising_edge (CLK) then
               if (diffdiff(31) = '1') then
                  off <= X"00000000" - diffdiff;
               else
                  off <= diffdiff;
               end if;
          end if;
     end process findDiff;
     
     offdiff <= std_logic_vector(off);

     DIFF <= offdiff(5 downto 0) & offdiff(31 downto 6);
     SIGN <= std_logic(diffdiff(31));

--     -- Find the sign of the time difference by taking the MSB. Note that if the
--     -- time difference is zero, we force the sign to zero... just because.
--     findSign : process (CLK) is
--     begin
--          if rising_edge (CLK) then
--               if (offdiff = X"00000000") then
--                  SIGN <= '0';
--               else
--                  SIGN <= std_logic(diffdiff(31));
--               end if;
--          end if;
--     end process findSign;

end architecture ark;






