library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

--------------------------------------------------------------------------------
-- Assume the two inputs times are in GPS seconds.
--
-- Outputs both a signed difference in two's complement (MSB is sign) and the
-- magnitude and sign information (for the Digital I/O display).
--
-- Ilya Belopolski
-- January 2012
--------------------------------------------------------------------------------

entity CalculateTimeDateDiff is port
   (
     CLK            : in    std_logic;
     PPS            : in    std_logic;
     INA            : in    std_logic_vector(31 downto 0);
     INB            : in    std_logic_vector(31 downto 0);
     DIFF           : out   std_logic_vector(31 downto 0);
     SIGNED_DIFF    : out   std_logic_vector(31 downto 0);
     SIGN           : out   std_logic
   );
end CalculateTimeDateDiff;


architecture ark of CalculateTimeDateDiff is

     signal CLK1         : std_logic_vector(31 downto 0);
     signal oldCLK1      : std_logic_vector(31 downto 0);
     signal CLK2         : std_logic_vector(31 downto 0);
     signal diffdiff     : signed(31 downto 0);
     signal off          : signed(31 downto 0);

begin

     -- Synchronize input signals to the 1PPS signal.
     CLK1 <= INA when rising_edge (PPS);
     oldCLK1 <= CLK1 when rising_edge (PPS);
     CLK2 <= INB when rising_edge (PPS);

     diffdiff <= signed(CLK2) - signed(oldCLK1);

     SIGNED_DIFF <= std_logic_vector(diffdiff);

     findDiff : process (CLK) is
     begin
          if rising_edge (CLK) then
               if (diffdiff(31) = '1') then
                  off <= X"00000000" - diffdiff;
               else
                  off <= diffdiff;
               end if;
          end if;
     end process findDiff;

     DIFF <= std_logic_vector(off);

     -- Output the sign of the time difference
     SIGN <= std_logic(diffdiff(31));

end architecture ark;
