library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

-- Convert GPS second-minute-hour-etc. time to GPS seconds
-- Assume GPS time (not UTC time or Hawaii time)

-- Only works for times after 2000..

entity ConvertToGPSSeconds is port
   (
     CLK            : in    std_logic;
     IRIG           : in    std_logic_vector(88 downto 0);
     GPS            : out   std_logic_vector(31 downto 0)
   );
end ConvertToGPSSeconds;


architecture ark of ConvertToGPSSeconds is

     -- Time at the year 2000... ah... those were the days.
     constant Y2KTime           : integer := 630720000;

     signal seconds             : integer;
     signal tenseconds          : integer;
     signal minutes             : integer;
     signal tenminutes          : integer;
     signal hours               : integer;
     signal tenhours            : integer;
     signal days                : integer;
     signal tendays             : integer;
     signal hundreddays         : integer;
     signal years               : integer;
     signal tenyears            : integer;

     signal yearDays            : integer;
     signal totalDays           : integer;
     signal totalHours          : integer;
     signal totalMinutes        : integer;
     signal totalSeconds        : integer;
     signal GPStime             : integer;
     signal totalYears          : integer;

     -- Use the lookup ROM (same as in IRIGB_encoder.Vhd) to find the GPS seconds at the start of each year
     component ROM_1024_32
         port (
             clka  : in  std_logic;
             addra : in  std_logic_vector (9 downto 0);
             douta : out std_logic_vector (31 downto 0)
         );
     end component;

     -- Signals for the lookup ROM second calculation
     signal addr               : std_logic_vector (9 downto 0);
     signal lookup             : std_logic_vector (31 downto 0);
--     signal totalYears         : unsigned (5 downto 0);

begin

     lookup_ROM : ROM_1024_32
         port map (
              clka  => CLK,
              addra => addr,
              douta => lookup
         );

     seconds     <= to_integer(unsigned(         IRIG(3  downto 0)));
     tenseconds  <= to_integer(unsigned((B"0" &  IRIG(7  downto 5))));
     minutes     <= to_integer(unsigned(         IRIG(11 downto 8)));
     tenminutes  <= to_integer(unsigned((B"0" &  IRIG(15 downto 13))));
     hours       <= to_integer(unsigned(         IRIG(20 downto 17)));
     tenhours    <= to_integer(unsigned((B"00" & IRIG(23 downto 22))));
     days        <= to_integer(unsigned(         IRIG(29 downto 26)));
     tendays     <= to_integer(unsigned(         IRIG(34 downto 31)));
     hundreddays <= to_integer(unsigned((B"00" & IRIG(36 downto 35))));
     years       <= to_integer(unsigned(         IRIG(47 downto 44)));
     tenyears    <= to_integer(unsigned(         IRIG(52 downto 49)));

     -- GPS years using lookup ROM
     totalYears <= years + 10*tenyears;
     addr <= std_logic_vector (to_unsigned(totalYears,6)) & B"0001";
     totalDays    <= days    + 10*tendays    + 100*hundreddays - 1;
     totalHours   <= hours   + 10*tenhours   + 24*totalDays;
     totalMinutes <= minutes + 10*tenminutes + 60*totalHours;
     totalSeconds <= seconds + 10*tenseconds + 60*totalMinutes;
     GPStime      <= totalSeconds + to_integer(unsigned(lookup)); -- Note: lookup carries the GPS time in seconds at the start of the year

        -- GPS years using straight calculation
--     totalYears <= years + 10*tenyears;
--     addr <= std_logic_vector (to_unsigned(totalYears,6)) & B"0001";
--
--     totalDays    <= days    + 10*tendays    + 100*hundreddays  - 1;
--     yearDays <= (365*3 + 366)*(totalYears/4) + (totalYears mod 4)*365;
--
--     totalDays    <= days    + 10*tendays    + 100*hundreddays + yearDays;
--     totalHours   <= hours   + 10*tenhours   + 24*totalDays;
--     totalMinutes <= minutes + 10*tenminutes + 60*totalHours;
--     totalSeconds <= seconds + 10*tenseconds + 60*totalMinutes;
--     GPStime      <= totalSeconds + Y2KTime;

     GPS <= std_logic_vector(to_unsigned(GPStime,32));

end architecture ark;
