-- Debounce the front-panel switches
--
-- Ilya Belopolski
-- 30 July, 2010


Library IEEE;
Use IEEE.Std_Logic_1164.all;
Use IEEE.Numeric_Std.all;

entity Debounce is port
   (
     CLK             : in    std_logic;
     RST             : in    std_logic;
     SW_IN           : in    std_logic;
     SW_OUT          : out   std_logic
   );
end Debounce;

architecture ark of Debounce is

   constant timeout_time : integer := 6000000; -- ~ 100ms

   signal sw_old           : std_logic;
   signal sw_ancient       : std_logic;
   signal change           : std_logic;
   signal timeout          : unsigned (31 downto 0);

begin

   -- Synchronize input
   syncinp: process (CLK, RST, SW_IN) is
   begin
         if RST = '1' then
            sw_old <= SW_IN;
            sw_ancient <= SW_IN;
         elsif rising_edge (CLK) then
            sw_old <= SW_IN;
            sw_ancient <= sw_old;
         end if;
   end process syncinp;

   -- Check for change
   change <= sw_old xor sw_ancient;

   -- Prevent more than one change from taking place in timeout_time;
   clean : process (CLK, RST, SW_IN) is
   begin
         if RST = '1' then
            SW_OUT <= SW_IN;
            timeout <= to_unsigned(timeout_time, 32);
         elsif rising_edge (CLK) then
            if (change = '1') and (timeout = 0) then
               SW_OUT <= sw_old;
               timeout <= to_unsigned(timeout_time, 32);
            elsif timeout /= 0 then
               timeout <= timeout - 1;
            end if;
         end if;
   end process clean;

end ark;
