--------------------------------------------------------------------------------
-- EEPROM Interface : a clock generator
-- Divide the OCXO clock (2^26 Hz = 67+MHz) by 14 to produce a clock of about (< 5MHz).
--
-- Ilya Belopolski
-- 16 August, 2010
--------------------------------------------------------------------------------
library IEEE;
Use IEEE.Std_Logic_1164.all;
Use IEEE.Numeric_Std.all;

entity EEPROMClock is port
   (
     CLK           : in    std_logic;
     HALFT         : in    std_logic_vector(31 downto 0);
     divCLK        : out   std_logic
   );
end EEPROMClock;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture ark of EEPROMClock is

   signal counter          : unsigned (31 downto 0);
   signal theCLK           : std_logic;

begin

     -- HALFT = 33,554,432
     -- This gives a PPS which is actually about one pulse per second.

     -- HALFT = 7
     -- This gives a clock just under 5MHz, the maximum for the EEPROM chip for
     -- the 2.5V at which it is operated by the IRIG-B daughter.


     -- Count clock cycles
     -- Whenever we count "HALFT" number of clock cycles, flip "divCLKC"
     genCLK : process (CLK) is
     begin
          if rising_edge (CLK) then
             if counter = unsigned(HALFT) and theCLK = '0' then
                counter <= X"00000000";
                theCLK <= '1';
             elsif counter = unsigned(HALFT) and theCLK = '1' then
                counter <= X"00000000";
                theCLK <= '0';
             else
                counter <= counter + 1;
             end if;
          end if;
     end process genCLK;

     divCLK <= theCLK;

end ark;
--------------------------------------------------------------------------------
