--------------------------------------------------------------------------------
-- SubModule LedDiag
--   CLK: FPGA clock, for example 2^26Hz = 67,108,864Hz
--        The LED clock will run at 128Hz
--   CNT[25..0]: Counter values for at 2^26Hz
--   GPS[31..0]: GPS time in sec
--   ONEPPS: One PPS signal which is used to synchronize the filter processing
--   LEDDIV: Divider for LED blink rate. Fundamental rate is 128Hz
--           0 - 0.5Hz; 1 - 1Hz; 2 - 2Hz; 3 - 4Hz; 4 - 8Hz; 5 - 16Hz;
--           6 - 32Hz; 7 - 64Hz; >=8 - constant on
--   SYNC: LED on/off toggle indicating the link is synchronizing
--   UP: LED on/off toggle indicating the link is up
-- Created   3/8/2016 9:15:02 AM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity LedDiag is port
   (
     CLK          : in    std_logic;
     CNT          : in    std_logic_vector(25 downto 0);
     GPS          : in    std_logic_vector(31 downto 0);
     ONEPPS       : in    std_logic;
     BLINK        : in    std_logic;
     DIV          : in    std_logic_vector(3 downto 0);
     SYNC         : out   std_logic;
     UP           : out   std_logic
   );
end LedDiag;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture RTL of LedDiag is

-- Signal Declarations
   signal count128           : unsigned (7 downto 0);
   signal mask128            : unsigned (7 downto 0);

begin
   -- 128 Hz count
   count128 <= unsigned (GPS(0 downto 0)) & unsigned (CNT(25 downto 19));
   -- 128 Hz mask
   getmask: process (BLINK, DIV, mask128) is
   begin
        if (BLINK = '1') then
        case DIV is
            when X"0" =>
               mask128 <= To_Unsigned (128, 8);
            when X"1" =>
               mask128 <= To_Unsigned (64, 8);
            when X"2" =>
               mask128 <= To_Unsigned (32, 8);
            when X"3" =>
               mask128 <= To_Unsigned (16, 8);
            when X"4" =>
               mask128 <= To_Unsigned (8, 8);
            when X"5" =>
               mask128 <= To_Unsigned (4, 8);
            when X"6" =>
               mask128 <= To_Unsigned (2, 8);
            when X"7" =>
               mask128 <= To_Unsigned (1, 8);
            when others =>
               mask128 <= To_Unsigned (0, 8);
         end case;
         else
               mask128 <= To_Unsigned (0, 8);
         end if;
   end process getmask;
   -- up
   UP <= '1' when ((not count128) and mask128) = To_Unsigned (0, 8) else '0';
   SYNC <= GPS(0);
end RTL;
--------------------------------------------------------------------------------
