--------------------------------------------------------------------------------
-- SubModule LinkRcv
-- Created   8/21/2008 2:34:45 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity LinkRcv is port
   (
     CLK           : in    std_logic;
     RST           : in    std_logic;
     MI            : in    std_logic;
     CRCERR        : in    std_logic;
     CRCOK         : in    std_logic;
     INADDR        : in    std_logic_vector(31 downto 0);
     INGPS         : in    std_logic_vector(31 downto 0);
     INA           : in    std_logic_vector(31 downto 0);
     INB           : in    std_logic_vector(31 downto 0);
     ADDR          : out   std_logic_vector(31 downto 0);
     GPS           : out   std_logic_vector(31 downto 0);
     A             : out   std_logic_vector(31 downto 0);
     B             : out   std_logic_vector(31 downto 0)
   );
end LinkRcv;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of LinkRcv is

-- Component Declarations

-- Type Declarations
   type state_type    is (BAD, GOOD);

-- Signal Declarations
   signal gpstime        : unsigned (31 downto 0);
   signal state          : state_type;

begin

   --- GPS clock; synchronize ADDR, A and B
   gps_clock: process (CLK) is
   begin
      if rising_edge (CLK) then
         if RST = '1' then
            gpstime <= (others => '0');
            ADDR <= (others => '0');
            A <= (others => '0');
            B <= (others => '0');
            state <= BAD;
         elsif MI = '1' then
            if state = GOOD then
               gpstime <= Unsigned (ingps) + 1;
               ADDR <= INADDR;
               A <= INA;
               B <= INB;
            else
               gpstime <= gpstime + 1;
            end if;
            state <= BAD;
         elsif CRCOK = '1' then
            state <= GOOD;
         end if;
      end if;
   end process gps_clock;
   GPS <= std_logic_vector(gpstime);

end Structure;
--------------------------------------------------------------------------------
