--------------------------------------------------------------------------------
-- Convert GPS vector into a display message
-- Ilya Belopolski
-- 15 August, 2010
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity WriteGPSMessage is port
   (
     RST        : in    std_logic;
     CLK        : in    std_logic;
     ONEPPS     : in    std_logic;
     GPS        : in    std_logic_vector(31 downto 0);
     MESS       : out   std_logic_vector(79 downto 0)
   );
end WriteGPSMessage;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture ark of WriteGPSMessage is

   signal onepps_old         : std_logic;
   signal onepps_trig        : std_logic;
   signal gps_bcd            : std_logic_vector(39 downto 0);
   signal writing            : std_logic_vector(79 downto 0);

-- Component declaration
   component BCDConv
      generic (
         M           : positive;      -- number of binary digits
         N           : positive       -- number of BCD digits
      );
      port (
         CLK         : in  std_logic;
         RST         : in  std_logic;
         INIT        : in  std_logic;  -- initialize conversion
         BUSY        : out std_logic;  -- indicates busy with conversion
         DIN         : in  std_logic_vector (M - 1 downto 0);    -- binary input
         DOUT        : out std_logic_vector (4 * N -1 downto 0); -- BCD result
         OVF         : out std_logic   -- overflow
      );
   end component;

begin

   onepps_old <= ONEPPS when rising_edge (CLK);
   onepps_trig <= ONEPPS and not onepps_old;

-- Binary to BCD conversion
   binBCDconvGPS: BCDConv
      generic map (
         M => 32,
         N => 10
      )
      port map (
         CLK => CLK,
         RST => RST,
         INIT => onepps_trig,
         BUSY => open,
         DIN => GPS,
         DOUT => gps_bcd,
         OVF => open
      );

   writing <= X"3" & gps_bcd (39 downto 36) &
              X"3" & gps_bcd (35 downto 32) &
              X"3" & gps_bcd (31 downto 28) &
              X"3" & gps_bcd (27 downto 24) &
              X"3" & gps_bcd (23 downto 20) &
              X"3" & gps_bcd (19 downto 16) &
              X"3" & gps_bcd (15 downto 12) &
              X"3" & gps_bcd (11 downto  8) &
              X"3" & gps_bcd (7  downto  4) &
              X"3" & gps_bcd (3  downto  0);

   MESS <= writing;


end architecture ark;
