--------------------------------------------------------------------------------
-- Convert leap second information into a display message.
-- Ilya Belopolski
-- 15-going-into-16 August, 2010
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity WriteLEAPMessage is port
   (
     CLK        : in    std_logic;
     RST        : in    std_logic;
     ONEPPS     : in    std_logic;
     LEAP       : in    std_logic_vector(7 downto 0);
     MESS       : out   std_logic_vector(79 downto 0)
   );
end WriteLEAPMessage;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture ark of WriteLEAPMessage is

   signal onepps_old         : std_logic;
   signal onepps_trig        : std_logic;
   signal leap_bcd           : std_logic_vector(7 downto 0);
   signal writing            : std_logic_vector(79 downto 0);

-- Component declaration
   component BCDConv
      generic (
         M           : positive;      -- number of binary digits
         N           : positive       -- number of BCD digits
      );
      port (
         CLK         : in  std_logic;
         RST         : in  std_logic;
         INIT        : in  std_logic;  -- initialize conversion
         BUSY        : out std_logic;  -- indicates busy with conversion
         DIN         : in  std_logic_vector (M - 1 downto 0);    -- binary input
         DOUT        : out std_logic_vector (4 * N -1 downto 0); -- BCD result
         OVF         : out std_logic   -- overflow
      );
   end component;

begin

   onepps_old <= ONEPPS when rising_edge (CLK);
   onepps_trig <= ONEPPS and not onepps_old;

-- Binary to BCD conversion
   binBCDconvLEAP: BCDConv
      generic map (
         M => 8,
         N => 2
      )
      port map (
         CLK => CLK,
         RST => RST,
         INIT => onepps_trig,
         BUSY => open,
         DIN => LEAP,
         DOUT => leap_bcd,
         OVF => open
      );

   writing <= X"20" & X"20" & X"20" & X"20" & X"20" &
              X"20" & X"3" & leap_bcd(7 downto 4) &
              X"3" & leap_bcd(3 downto 0) & X"20" & X"20";

   MESS <= writing;

end architecture ark;


