--------------------------------------------------------------------------------
-- Convert time zone information into a display message
-- Ilya Belopolski
-- 15 August, 2010
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity WriteTZMessage is port
   (
     CLK        : in    std_logic;
     RST        : in    std_logic;
     ONEPPS     : in    std_logic;
     TZ_OFFSET  : in    std_logic_vector(5 downto 0);
     TZ_SIGN    : in    std_logic;
     MESS       : out   std_logic_vector(79 downto 0)
   );
end WriteTZMessage;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture ark of WriteTZMessage is

   signal onepps_old         : std_logic;
   signal onepps_trig        : std_logic;
   signal tz_bcd             : std_logic_vector(7 downto 0);
   signal writing            : std_logic_vector(79 downto 0);

   signal sign               : std_logic_vector(7 downto 0);
   signal tz_even            : std_logic_vector(5 downto 0);
   signal hours_calc         : unsigned(5 downto 0);
   signal hours              : std_logic_vector(4 downto 0);
   signal tenths             : std_logic_vector(7 downto 0);

-- Component declaration
   component BCDConv
      generic (
         M           : positive;      -- number of binary digits
         N           : positive       -- number of BCD digits
      );
      port (
         CLK         : in  std_logic;
         RST         : in  std_logic;
         INIT        : in  std_logic;  -- initialize conversion
         BUSY        : out std_logic;  -- indicates busy with conversion
         DIN         : in  std_logic_vector (M - 1 downto 0);    -- binary input
         DOUT        : out std_logic_vector (4 * N -1 downto 0); -- BCD result
         OVF         : out std_logic   -- overflow
      );
   end component;

begin

   onepps_old <= ONEPPS when rising_edge (CLK);
   onepps_trig <= ONEPPS and not onepps_old;

   parseOffset : process (CLK) is
   begin
        if rising_edge (CLK) then
           if (TZ_OFFSET(0) = '1') then
              tenths <= X"35";
           else
              tenths <= X"30";
           end if;
        end if;
   end process parseOffset;

   tz_even <= TZ_OFFSET(5 downto 1) & B"0";
   hours_calc <= unsigned(tz_even)/2;
   hours <= std_logic_vector(hours_calc(4 downto 0));

-- Binary to BCD conversion
   binBCDconvTZ: BCDConv
      generic map (
         M => 5,
         N => 2
      )
      port map (
         CLK => CLK,
         RST => RST,
         INIT => onepps_trig,
         BUSY => open,
         DIN => hours,
         DOUT => tz_bcd,
         OVF => open
      );

   chooseSign : process (CLK) is
   begin
        if rising_edge (CLK) then
           if (TZ_SIGN = '1') then
              sign <= X"2D";
           else
              sign <= X"20";
           end if;
        end if;
   end process chooseSign;

   writing <= X"20" & X"20" & X"20" & X"20" &
              sign & X"3" & tz_bcd(7 downto 4) &
              X"3" & tz_bcd(3 downto 0) &
              X"20" & tenths & X"20";

   MESS <= writing;


end architecture ark;
